<? /*
 * phpHartPOP - A simple web interface to a POP3 mailbox
 * Copyright (C) 2003 Sebastian Pipping
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Sebastian Pipping <webmaster@hartwork.org>
 */

define( MIME_DEBUG, FALSE );
define( DEFAULT_DATE, "Y-m-d H:i:s" );
define( MAX_LEN_TO,       30 );
define( MAX_LEN_SUBJECT,  40 );

DEFINE( FLAG_FROM,     1 );
DEFINE( FLAG_TO,       2 );
DEFINE( FLAG_SUBJECT,  4 );
DEFINE( FLAG_DATE,     8 );

// ---------------------------------------------------------------------------------------------------------get_from-----
function get_from( &$header, &$out_name, &$out_addy )
{
	$matches = array();
	if( preg_match( "/([fF]rom|FROM): *([^\r\n]+)/", $header, $matches ) )
	{
		if( MIME_DEBUG ) echo "FROM: [" . $matches[ 2 ] . "]\n";
		get_both( $matches[ 2 ], $out_name, $out_addy );
	}
	else if( preg_match( "/([rR]eturn-[pP]ath|RETURN-PATH): *([^\r\n]+)/", $header, $matches ) )
	{
		if( MIME_DEBUG ) echo "FROM: [" . $matches[ 2 ] . "]\n";
		get_both( $matches[ 2 ], $out_name, $out_addy );
	}
	else if( preg_match( "/([rR]eply-[tT]o|REPLY-TO): *([^\r\n]+)/", $header, $matches ) )
	{
		if( MIME_DEBUG ) echo "FROM: [" . $matches[ 2 ] . "]\n";
		get_both( $matches[ 2 ], $out_name, $out_addy );
	}
	else
	{
		if( MIME_DEBUG ) echo "FROM: ERROR\n";
		$out_name = "unknown";
		$out_addy = "un@known.com";
	}
}

// -----------------------------------------------------------------------------------------------------------get_to-----
function get_to( &$header, &$out_to_names, &$out_to_addies )
{
	$matches = array();
	$out_to_names = array();
	$out_to_addies = array();
	if( preg_match( "/To: *([^\r\n]+)/", $header, $matches ) )
	{
		if( MIME_DEBUG ) echo "TO: [" . $matches[ 1 ] . "]\n";

		if( preg_match( "/,$/", $matches[ 1 ] ) )
		{
			if( MIME_DEBUG ) echo "TO: SEVERAL\n";

			// Several rcpts
			if( preg_match( "/To: *(([^,\r\n]+,[\r\n\t ]+)+[^\r\n]+)/", $header, $matches ) )
			{
				$array = preg_split( "/,[ \r\n\t]*/", $matches[ 1 ] );
				$alen = count( $array );
				for( $i = 0; $i < $alen; $i++ )
				{
					$out_to_names[ $i ] = "";
					$out_to_addies[ $i ] = "";
					get_both( $array[ $i ], $out_to_names[ $i ], $out_to_addies[ $i ] );
				}
			}
			else
			{
				if( MIME_DEBUG ) echo "TO: ERROR\n";
				$out_to_names[ 0 ] = "unknown";
				$out_to_addies[ 0 ] = "un@known.com";
			}
		}
		else
		{
			if( MIME_DEBUG ) echo "TO: SINGLE\n";

			// Single rcpt
			$out_to_names[ 0 ] = "";
			$out_to_addies[ 0 ] = "";
			get_both( $matches[ 1 ], $out_to_names[ 0 ], $out_to_addies[ 0 ] );
		}
	}
	else
	{
		if( MIME_DEBUG ) echo "TO: MISSING\n";
		$out_to_names[ 0 ] = "unknown";
		$out_to_addies[ 0 ] = "un@known.com";
	}
}

// ---------------------------------------------------------------------------------------------------------get_both-----
function get_both( &$header_line, &$out_name, &$out_addy )
{
	$matches = array();
	if( preg_match( "/^([\"']*([^\"']*)[\"']* +)?<([^>]*)>( +\((.+)\))?$/", $header_line, $matches ) )
	{
		if( MIME_DEBUG ) echo "Case ALPHA\n";

		// Catches
		// A: "name" <user@host.ext>
		// B: name <user@host.ext>
		// C: name <>
		// D: <user@host.ext>
		// E: <user@host.ext> (name)
		// F: "" <user@host.ext>

		// Address
		if( $matches[ 3 ] != "" )
		{
			$out_addy = $matches[ 3 ];
		}
		else
		{
			// Example: [From: CompuServe Kundenbetreuung <>]
			$out_addy = "un@known.com";
		}

		// Name
		if( $matches[ 2 ] != "" )
		{
			$out_name = $matches[ 2 ];
		}
		else if( $matches[ 5 ] != "" )
		{
			$out_name = $matches[ 5 ];
		}
		else
		{
			$out_name = $out_addy;
		}
	}
	else if( preg_match( "/^([^ <>]+)( +\((.+)\))?$/", $header_line, $matches ) )
	{
		if( MIME_DEBUG ) echo "Case BETA\n";
		// Catches
		// G: user@host.ext
		// H: user@host.ext (name)

		// Address -- @host1.ext1:user@host2.ext2
		$out_addy = preg_replace( "/^.+:/", "", $matches[ 1 ] );

		// Name
		if( $matches[ 3 ] != "" )
		{
			$out_name = $matches[ 3 ];
		}
		else
		{
			$out_name = $out_addy;
		}
	}
	else if( preg_match( "/^([^<>]+)<([^<>]+)>$/", $header_line, $matches ) )
	{
		if( MIME_DEBUG ) echo "Case GAMMA\n";
		// Catches
		// I: name<user@host.ext>

		// Address
		$out_addy = $matches[ 2 ];

		// Name
		$out_name = $matches[ 1 ];
	}
	else if( preg_match( "/^([\"']*([^\"']+)[\"']* +)?([^>]+)>( +\\((.+)\\))?$/", $header_line, $matches ) )
	{
   		if( MIME_DEBUG ) echo "Case DELTA\n";
		// Catches
		// I: "name" user@host.ext>
		// J: name user@host.ext>
		// K: user@host.ext> (name)


		// Address
		$out_addy = $matches[ 3 ];

		// Name
		if( $matches[ 2 ] != "" )
		{
			$out_name = $matches[ 2 ];
		}
		else if( $matches[ 5 ] != "" )
		{
			$out_name = $matches[ 5 ];
		}
		else
		{
			$out_name = $out_addy;
		}
	}
	else
	{
		if( MIME_DEBUG ) echo "Case ERROR\n";
		$out_name = "unknown";
		$out_addy = "un@known.com";
	}

	resolve_iso( $out_name );

	// Kick ["], ['] and [ ]
	$out_name = preg_replace( "/^[ '\"]+/", "", $out_name );
	$out_name = preg_replace( "/[ '\"]+$/", "", $out_name );

	// Convert "\(..\)" to "(..)"
	$out_name = preg_replace( "/\\\\\((.+)\\\\\)/", "(\\1)", $out_name );

	// Strip ["] and [']
	//	$out_name = preg_replace( "/([\"'])([^\\1])\\1/", "\\2", $out_name );
}

// ------------------------------------------------------------------------------------------------------get_subject-----
function get_subject( &$header, &$out_subject, $max_len = MAX_LEN_SUBJECT )
{
	$matches = array();
	if( preg_match( "/([sS]ubject|SUBJECT): ([^\r\n]+)/", $header, $matches ) )
	{
		$out_subject = preg_replace( "/  +/", " ", $matches[ 2 ] );
		$out_subject = preg_replace( "/ +$/", "", $out_subject );

		resolve_iso( $out_subject );
		$out_subject = trim( $out_subject );

		if( strlen( $out_subject ) > $max_len )
		{
			$out_subject = substr( $out_subject, 0, $max_len - 3 ) . "...";
		}
	}
	else
	{
		$out_subject = "no subject";
	}
}

// ---------------------------------------------------------------------------------------------------------get_date-----
function get_date( &$header, &$out_date, $format = DEFAULT_DATE )
{
	$matches = array();
	if( preg_match( "/Date: ([^\r\n]+)/", $header, $matches ) )
	{
		if( MIME_DEBUG ) echo "\nDATE: " . $matches[ 1 ] . "\n";

		// Spelling correction
		// $matches[ 1 ] = preg_replace( "/ ug /", " Aug ", $matches[ 1 ] );

		$out_date = @date( $format, strtotime( $matches[ 1 ] ) );
		if( $out_date == "" )
		{
			if( MIME_DEBUG ) echo "\nDATE: STRANGE\n";
			$out_date = preg_replace( "/\d/", "0", date( $format, time() ) );
		}
	}
	else
	{
		if( MIME_DEBUG ) echo "\nDATE: MISSING\n";
		$out_date = preg_replace( "/\d/", "0", date( $format, time() ) );
	}
}

// ----------------------------------------------------------------------------------------------------resolve_chars-----
function resolve_chars( $matches )
{
	return chr( base_convert( $matches[ 1 ], 16, 10 ) );
}

// ------------------------------------------------------------------------------------------------------resolve_iso-----
function resolve_iso( &$text )
{
	$matches = array();
	while( preg_match( "/(.*)=\?((iso|ISO)-8859-1|[wW]indows-1252)\?[qQ]\?(.+)\?=(.*)/", $text, $matches ) )
	{
		$pre = $matches[ 1 ];
		$post = $matches[ 5 ];

		$iso = str_replace( "_", " ", $matches[ 4 ] );
		$iso = preg_replace_callback( "/=(.{2})/", resolve_chars, $iso );

		$text = $pre . $iso . $post;
	}
	while( preg_match( "/(.*)=\?(iso|ISO)-8859-1\?[bB]\?(.+)=?\?=(.*)/", $text, $matches ) )
	{
		$pre = $matches[ 1 ];
		$post = $matches[ 4 ];

		$iso = base64_decode( $matches[ 3 ] );

		$text = $pre . $iso . $post;
	}
}

?>